import 'package:flutter/material.dart';
import 'package:flutter/widgets.dart';

enum PaintPosition { Top, Bottom }

class WavePainter extends CustomPainter {
  final double height;
  final double width;
  final int wave;
  final double lowFactor;
  final double highFactor;
  final Color startColor;
  final Color endColor;
  final PaintPosition position;
  WavePainter(
      {@required this.height,
      @required this.width,
      @required this.wave,
      this.lowFactor = 0.5,
      this.highFactor = 1,
      this.startColor = Colors.blueAccent,
      this.endColor = Colors.purpleAccent,
      this.position = PaintPosition.Top});
  @override
  void paint(Canvas canvas, Size size) {
    if (this.position == PaintPosition.Top) {
      _paintTop(canvas, size);
    } else {
      _paintBottom(canvas, size);
    }
  }

  void _paintTop(Canvas canvas, Size size) {
    var paint = Paint();
    var path = Path();
    final partWidth = width / (wave * 2);
    final partHeight = height;
    path.moveTo(0, partHeight * lowFactor);
    for (int i = 0; i <= wave; i++) {
      final w1 = partWidth * i * 2 + partWidth;
      final h1 = partHeight * (i % 2 == 0 ? lowFactor : highFactor);
      final h2 = partHeight * (i % 2 == 0 ? highFactor : lowFactor);
      final w2 = partWidth * (i + 1) * 2;
      path.cubicTo(w1, h1, w1, h2, w2, h2);
    }
    path.lineTo(width, 0);
    path.lineTo(0, 0);

    path.close();

    paint.shader = LinearGradient(
            colors: [this.startColor, this.endColor],
            tileMode: TileMode.repeated,
            begin: Alignment.topLeft,
            end: Alignment.bottomRight)
        .createShader(Rect.fromLTRB(0, 0, width, partHeight));

    paint.style = PaintingStyle.fill;
    paint.blendMode = BlendMode.multiply;
    canvas.drawPath(path, paint);
  }

  void _paintBottom(Canvas canvas, Size size) {
    var paint = Paint();
    var path = Path();
    final partWidth = width / (wave * 2);
    final partHeight = height;
    path.moveTo(0, height - partHeight * lowFactor);
    for (int i = 0; i <= wave; i++) {
      final w1 = partWidth * i * 2 + partWidth;
      final h1 = height - partHeight * (i % 2 == 0 ? lowFactor : highFactor);
      final h2 = height - partHeight * (i % 2 == 0 ? highFactor : lowFactor);
      final w2 = partWidth * (i + 1) * 2;
      path.cubicTo(w1, h1, w1, h2, w2, h2);
    }
    path.lineTo(width, height);
    path.lineTo(0, height);

    path.close();

    paint.shader = LinearGradient(
            colors: [this.startColor, this.endColor],
            tileMode: TileMode.repeated,
            begin: Alignment.topLeft,
            end: Alignment.bottomRight)
        .createShader(Rect.fromLTRB(0, 0, width, partHeight));

    paint.style = PaintingStyle.fill;
    paint.blendMode = BlendMode.multiply;
    canvas.translate(0, size.height);
    canvas.drawPath(path, paint);
  }

  @override
  bool shouldRepaint(CustomPainter oldDelegate) {
    return false;
  }
}
