import 'package:flutter/material.dart';
import 'package:flutter/widgets.dart';

class Button extends StatefulWidget {
  final Color color;
  final String text;
  final Function onPressed;
  final bool enabled;
  final bool busy;
  final Color textColor;
  final double width;
  final double height;
  final bool visible;
  final String busyText;
  final double minWidth;
  final double cornerRadius;
  Button(
      {@required this.text,
      @required this.onPressed,
      this.color = Colors.blueAccent,
      this.enabled = true,
      this.busy = false,
      this.textColor = Colors.white,
      this.width = double.maxFinite,
      this.minWidth = double.minPositive,
      this.height = 40,
      this.visible = true,
      this.busyText = "Loading...",
      this.cornerRadius = 2
      });
  @override
  State<StatefulWidget> createState() {
    return _ButtonState();
  }
}

class _ButtonState extends State<Button> {
  @override
  Widget build(BuildContext context) {
    return AnimatedContainer(
      duration: Duration(milliseconds: 200),
      child: widget.visible
          ? FlatButton(
              shape: RoundedRectangleBorder(
                  borderRadius: BorderRadius.circular(this.widget.cornerRadius)),
              onPressed: widget.enabled ? widget.onPressed : null,
              disabledColor: Colors.grey.withAlpha(0x80),
              textColor: widget.textColor,
              child: Text(widget.busy ? widget.busyText : widget.text),
              color: widget.color,
            )
          : Container(),
      decoration: BoxDecoration(
          boxShadow: widget.enabled && widget.visible
              ? <BoxShadow>[
                  BoxShadow(
                    color: widget.color.withAlpha(0x50),
                    blurRadius: 16,
                  )
                ]
              : []),
      constraints:
          BoxConstraints(maxWidth: widget.width, minHeight: widget.height,
          minWidth: widget.minWidth),
    );
  }
}
