import 'package:flutter/material.dart';

class ValidatableTextField extends StatefulWidget {
  final String title;
  final Function(String) textChanged;
  final String currentValue;
  final TextInputType type;
  final bool isPassword;
  final Color color;
  final Color textColor;
  final String placeHolder;
  final Color shadowColor;
  final Color labelColor;
  final Color labelTextColor;
  final Color errorColor;
  final String error;
  final bool isError;
  ValidatableTextField(
      {@required this.title,
      @required this.textChanged,
      @required this.currentValue,
      this.type = TextInputType.text,
      this.isPassword = false,
      this.color = Colors.white,
      this.textColor = Colors.black54,
      this.placeHolder = "",
      this.shadowColor = const Color(0xff757de8),
      this.labelColor = const Color(0xffe8eaf6),
      this.labelTextColor = const Color(0xff757de8),
      this.isError = false,
      this.error = "",
      this.errorColor = const Color.fromARGB(255, 244, 67, 54)});
  @override
  State<StatefulWidget> createState() =>
      _ValidatableTextFieldState(TextEditingController(text: currentValue));
}

class _ValidatableTextFieldState extends State<ValidatableTextField> {
  final TextEditingController controller;
  _ValidatableTextFieldState(this.controller);
  @override
  Widget build(BuildContext context) {
    return Container(
        constraints: BoxConstraints(maxWidth: 400),
        child: Padding(
            padding: EdgeInsets.only(top: 4, bottom: 4),
            child: Column(
              mainAxisAlignment: MainAxisAlignment.start,
              crossAxisAlignment: CrossAxisAlignment.stretch,
              children: <Widget>[
                Align(
                  child: Transform.translate(
                    offset: Offset(0, 4),
                    child: Container(
                        decoration: BoxDecoration(
                            color: widget.labelColor,
                            borderRadius: BorderRadius.only(
                                topLeft: Radius.circular(4),
                                topRight: Radius.circular(4))),
                        child: Padding(
                          padding: EdgeInsets.only(
                              left: 8, right: 8, top: 1, bottom: 8),
                          child: Text(
                            widget.title,
                            style: TextStyle(
                                inherit: true, color: widget.labelTextColor),
                          ),
                        )),
                  ),
                  alignment: Alignment.centerLeft,
                ),
                Container(
                  decoration: BoxDecoration(
                      color: widget.color,
                      borderRadius: BorderRadius.only(
                          bottomLeft: Radius.circular(widget.isError ? 0 : 4),
                          topRight: Radius.circular(4),
                          bottomRight: Radius.circular(4)),
                      boxShadow: [
                        BoxShadow(
                          color: widget.shadowColor.withOpacity(0.25),
                          blurRadius:
                              4, // has the effect of softening the shadow
                          spreadRadius:
                              1, // has the effect of extending the shadow
                          offset: Offset(
                            1, // horizontal, move right 10
                            4, // vertical, move down 10
                          ),
                        )
                      ]),
                  child: TextField(
                    decoration: InputDecoration(
                      focusedBorder: InputBorder.none,
                      hintText: widget.placeHolder,
                      hintStyle: TextStyle(
                          inherit: true,
                          color: widget.textColor.withOpacity(0.8)),
                      border: InputBorder.none,
                      contentPadding: EdgeInsets.fromLTRB(8, 12, 8, 12),
                    ),
                    style: TextStyle(inherit: true, color: widget.textColor),
                    onChanged: widget.textChanged,
                    obscureText: widget.isPassword,
                    cursorColor: widget.textColor,
                    cursorRadius: Radius.circular(8),
                    keyboardType: widget.type,
                  ),
                ),
                Align(
                  child: Transform.translate(
                    offset: Offset(0, 0),
                    child: AnimatedContainer(
                        duration: Duration(milliseconds: 200),
                        decoration: BoxDecoration(
                            color: widget.labelColor,
                            borderRadius: BorderRadius.only(
                                bottomLeft: Radius.circular(4),
                                bottomRight: Radius.circular(4)),
                            boxShadow: [
                              BoxShadow(
                                color: widget.shadowColor.withOpacity(0.25),
                                blurRadius:
                                    4, // has the effect of softening the shadow
                                spreadRadius:
                                    1, // has the effect of extending the shadow
                                offset: Offset(
                                  1, // horizontal, move right 10
                                  4, // vertical, move down 10
                                ),
                              )
                            ]),
                        child: widget.isError
                            ? Padding(
                                padding: EdgeInsets.only(
                                    left: 4, right: 4, top: 4, bottom: 8),
                                child: Text(
                                  widget.error,
                                  style: TextStyle(
                                      inherit: true,
                                      color: widget.errorColor,
                                      fontSize: 12),
                                ))
                            : null),
                  ),
                  alignment: Alignment.centerLeft,
                ),
              ],
            )));
  }

  @override
  void dispose() {
    // Clean up the controller when the widget is removed from the
    // widget tree.
    controller.dispose();
    super.dispose();
  }
}
